/*

    UtilityMeterLogger

    Detects pulses on analogue pins and logs pulses-per-minute counts to SD card

    Board: Arduino Nano (old bootloader)
    Programmer: Micronucleus on /dev/ttyUSB0

*/


#include <SPI.h>
#include <SD.h>
#include <OneWire.h>
#include <DallasTemperature.h>

#define ADCPERIOD     20       // milliseconds between ADC samples
#define LOGINTERVAL   3000 //60*1000/ADCPERIOD   // ADC-sample-periods between log records (60 sec)

#define THRESHL   500             // go below this to register a pulse (in ADC units)
#define THRESHH   700             // go above this to reset the pulse detection
#define STATEHI   true
#define STATELO   false
#define SENSOR0PIN 0    // analog input A0, gas meter pulse sensor
#define SENSOR1PIN 3    // digital input D3, elec meter pulse sensor
#define TEMPPIN    7    // digital input D7, OneWire temperature sensors


const int SDchipSelect = 2; // pin D2
volatile byte ms_counter = ADCPERIOD;
volatile unsigned long sample_counter = 0;
volatile bool ch0state = STATEHI; // gas meter sensor state
volatile bool ch1state = STATELO; // elec meter sensor state
bool SDready = false;
int devicecount = 0;
float tempC;

OneWire  ds(TEMPPIN);  // on pin 10 (a 4.7K resistor is necessary)
DallasTemperature sensors(&ds);

/*
 * Setup
 */
void setup() {
  // Open serial communications and wait for port to open:
  Serial.begin(9600);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for native USB port only
  }

  
  Serial.print("Initializing SD card...");
  // see if the card is present and can be initialized:
  SDready = SD.begin(SDchipSelect);
  if (!SDready) {
    Serial.println("Card failed, or not present. Continuing without logging.");
  }
  else {
    Serial.println("card initialized.");
  }

  // Start up the temp sensor library
  sensors.begin();
  devicecount = sensors.getDeviceCount();
  sensors.setResolution(12);
  Serial.println("Found "+String(devicecount)+" temperature sensors.");


  // Timer0 is already used for millis() - we'll just interrupt somewhere
  // in the middle and call the "Compare A" function below
  OCR0A = 0xAF;
  TIMSK0 |= _BV(OCIE0A);
}

/*
 * Interrupt, called once a millisecond
 */
SIGNAL(TIMER0_COMPA_vect)
{
  ms_counter--; //increment a 1ms-per-tick counter (which rolls over at 255)
}

/*
 * Edge detection for ch0 (gas meter sensor)
 */
bool ch0pulseOccurred() {
  bool retval = false;
  unsigned int ch0val = analogRead(SENSOR0PIN);

  if ((ch0state == STATEHI) & (ch0val < THRESHL)) {
    retval = true; // register a pulse
    ch0state = STATELO;
    Serial.println("Gas: pulse detected.");
  }
  else if ((ch0state == STATELO) & (ch0val > THRESHH)) {
    ch0state = STATEHI;
  }
  return retval;
}

/*
 * Edge detection for ch1 (elec meter sensor)
 */
bool ch1pulseOccurred() {
  bool retval = false;
  bool ch1val = digitalRead(SENSOR1PIN);

  if ((ch1state == STATELO) & (ch1val)) {
    retval = true; // register a pulse
    ch1state = STATEHI;
    Serial.println("Elec: pulse detected.");
  }
  else if ((ch1state == STATEHI) & !ch1val) {
    ch1state = STATELO;
  }
  return retval;
}




unsigned int ch0pulse_count = 0;
unsigned int ch1pulse_count = 0;
String dataString = "";
unsigned long lognum = 0;

/*
 * Main loop.
 */
void loop() {


  // check for '0' on dial (do this less often because it happens slowly)
  if (ms_counter == 0) {
    ms_counter = ADCPERIOD;
    sample_counter ++;

    if (ch0pulseOccurred()) {
      ch0pulse_count++;
    }

   // check for LED flashes (do this often because the flashes are short)
   if (ch1pulseOccurred()) {
      ch1pulse_count++;
    }

   // Serial.println(String(sample_counter));
  }


  if (sample_counter >= LOGINTERVAL) {
    // time to write a log record
    sample_counter = 0;

    // request sensor readings
    devicecount = sensors.getDeviceCount();
    sensors.requestTemperatures();

    // build logfile record string
    dataString = String(lognum++)+","+String(ch0pulse_count)+","+String(ch1pulse_count);

    for (int sdex = 0; sdex < devicecount; sdex++)
    {
      tempC = sensors.getTempCByIndex(sdex);
      dataString += ","+String(tempC);
//      Serial.println("Sensor "+String(sdex)+" : "+String(tempC)+"C");
    }

    // print to serial port
    Serial.println(dataString);

    // if the file is available, write to it:
    if (SDready) {
      // open the file. note that only one file can be open at a time,
      // so you have to close this one before opening another.
      File dataFile = SD.open("datalog.txt", FILE_WRITE);

      if (dataFile) {
        dataFile.println(dataString);
        dataFile.close();
      }
      // if the file isn't open, pop up an error:
      else {
        Serial.println("error opening datalog.txt");
      }
    }


    ch0pulse_count = 0;
    ch1pulse_count = 0;
  }
}
